package com.dzeio.openhealth.ui.browse

import android.Manifest
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.navigation.fragment.findNavController
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentBrowseBinding
import com.dzeio.openhealth.utils.PermissionsManager
import com.google.android.material.card.MaterialCardView
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class BrowseFragment :
    BaseFragment<BrowseViewModel, FragmentBrowseBinding>(BrowseViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentBrowseBinding
        get() = FragmentBrowseBinding::inflate

    private lateinit var button: MaterialCardView
    private val activityResult = registerForActivityResult(
        ActivityResultContracts.RequestMultiplePermissions()
    ) {
        if (it.containsValue(false)) {
            // TODO: Show a popup with choice to change it
            Toast.makeText(requireContext(), R.string.permission_declined, Toast.LENGTH_LONG).show()
            return@registerForActivityResult
        }

        button.callOnClick()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // handle clicking on the weight card
        binding.weight.setOnClickListener {
            findNavController().navigate(BrowseFragmentDirections.actionNavBrowseToNavListWeight())
        }

        // handle clicking on the water intake card
        binding.waterIntake.setOnClickListener {
            findNavController().navigate(BrowseFragmentDirections.actionNavBrowseToNavWaterHome())
        }

        // handle clicking on the food calories card
        binding.foodCalories.setOnClickListener {
            findNavController().navigate(BrowseFragmentDirections.actionNavBrowseToFoodHomeFragment())
        }

        // handle clicking on the steps card
        binding.steps.setOnClickListener {
            // since Android Q We need additionnal permissions
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
                // check for activity permission
                val activityPermission = PermissionsManager.hasPermission(
                    requireContext(),
                    Manifest.permission.ACTIVITY_RECOGNITION
                )

                // check for notification permission
                val notificationPermission =
                    Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU && PermissionsManager.hasPermission(
                        requireContext(),
                        Manifest.permission.POST_NOTIFICATIONS
                    )

                val permissionsToAsk = arrayListOf<String>()

                // add missing permission to list
                if (!activityPermission) {
                    permissionsToAsk.add(Manifest.permission.ACTIVITY_RECOGNITION)
                }

                // add missing permission to list only if necessary
                if (!notificationPermission && Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                    permissionsToAsk.add(Manifest.permission.POST_NOTIFICATIONS)
                }

                // ask for permissions
                if (permissionsToAsk.isNotEmpty()) {
                    button = binding.steps
                    activityResult.launch(permissionsToAsk.toTypedArray())
                    return@setOnClickListener
                }
            }

            // navigate user to the Steps home fragment
            findNavController().navigate(
                BrowseFragmentDirections.actionNavBrowseToStepsHomeFragment()
            )
        }

        // display the number of steps the user made today
        viewModel.steps.observe(viewLifecycleOwner) {
            updateStepsText(it, viewModel.stepsGoal.value)
        }

        // display the number of steps the user should do today
        viewModel.stepsGoal.observe(viewLifecycleOwner) {
            updateStepsText(viewModel.steps.value, it)
        }

        // display the current user's weight
        viewModel.weight.observe(viewLifecycleOwner) {
            binding.weightText.setText(
                String.format(
                    resources.getString(R.string.weight_current),
                    String.format(resources.getString(R.string.unit_mass_kilogram_unit), it)
                )
            )
        }

    }

    private fun updateStepsText(numberOfSteps: Int?, goal: Int?) {
        var text = "${numberOfSteps ?: 0} steps"
        if (goal != null) {
            text = "${numberOfSteps ?: 0} of $goal steps"
        }
        binding.stepsText.setText(text)
    }
}
