package com.dzeio.openhealth.ui.home

import android.animation.ValueAnimator
import android.content.SharedPreferences
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.RectF
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.FragmentHomeBinding
import com.dzeio.openhealth.graphs.WeightChart
import com.dzeio.openhealth.ui.weight.WeightDialog
import com.dzeio.openhealth.units.Units
import com.dzeio.openhealth.utils.DrawUtils
import com.dzeio.openhealth.utils.GraphUtils
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import kotlin.math.max
import kotlin.math.min

@AndroidEntryPoint
class HomeFragment : BaseFragment<HomeViewModel, FragmentHomeBinding>(HomeViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentHomeBinding
        get() = FragmentHomeBinding::inflate

    private val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onStart() {
        super.onStart()
        // Bindings
        binding.addWeight.setOnClickListener {
            findNavController().navigate(
                HomeFragmentDirections.actionNavHomeToNavWeightDialog(
                    WeightDialog.DialogTypes.ADD_WEIGHT.ordinal
                )
            )
        }

        /**
         * Water Intake
         */

        binding.fragmentHomeWaterAdd.setOnClickListener {
            val water = viewModel.water.value
            if (water == null || !water.isToday()) {
                val w = Water()
                w.value = viewModel.waterCupSize.value ?: 200
                viewModel.updateWater(w)
            } else {
                water.value += viewModel.waterCupSize.value ?: 200
                viewModel.updateWater(water)
            }
        }
        val waterUnit =
            Units.Volume.find(settings.getString("water_unit", "milliliter") ?: "Milliliter")

        binding.fragmentHomeWaterTotal.text =
            String.format(
                resources.getString(waterUnit.unit),
                viewModel.dailyWaterIntake
            )

        binding.fragmentHomeWaterRemove.setOnClickListener {
            val water = viewModel.water.value
            if (water != null) {
                water.value -= viewModel.waterCupSize.value ?: 200
                if (water.value <= 0) {
                    viewModel.deleteWater(water)
                } else {
                    viewModel.updateWater(water)
                }
            }
        }

        // handle button to go to weight home
        binding.listWeight.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavListWeight())
        }

        // handle button to go to water intake home
        binding.gotoWaterHome.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavWaterHome())
        }

        // Make a line Chart using the graph library
        GraphUtils.lineChartSetup(
            binding.weightGraph,
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorPrimary
            ),
            MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnBackground
            )
        )

        // Update the water intake Graph when the water intake changes
        viewModel.water.observe(viewLifecycleOwner) {
            if (it != null) {
                updateWater(it.value)
            } else {
                updateWater(0)
            }
        }

        // Update the steps Graph when the steps count changes
        viewModel.steps.observe(viewLifecycleOwner) {
            binding.stepsCurrent.text = it.toString()
        }

        // Update the steps Graph when the goal changes
        viewModel.stepsGoal.observe(viewLifecycleOwner) {
            if (it == null) {
                binding.stepsTotal.text = ""
                return@observe
            }
            binding.stepsTotal.text = it.toString()
        }

        // update the graph when the weight changes
        viewModel.weights.observe(viewLifecycleOwner) {
            if (it != null) {
                updateGraph(it)
            }
        }

        // update the graph when the goal weight change
        viewModel.goalWeight.observe(viewLifecycleOwner) {
            if (viewModel.weights.value != null) updateGraph(viewModel.weights.value!!)
        }

        // update the graph when the weight unit change
        viewModel.massUnit.observe(viewLifecycleOwner) {
            if (viewModel.weights.value != null) updateGraph(viewModel.weights.value!!)
        }
    }

    /**
     * Function that update the graph for the weight
     */
    private fun updateGraph(list: List<Weight>) {
        WeightChart.setup(
            binding.weightGraph,
            requireView(),
            list,
            viewModel.massUnit.value!!,
            viewModel.goalWeight.value
        )
    }

    /**
     * the waterintake old value to keep for value update
     */
    private var oldValue = 0f

    /**
     * function that update the water count in the home page
     */
    private fun updateWater(newValue: Int) {

        // get the current Unit
        val waterUnit =
            Units.Volume.find(settings.getString("water_unit", "milliliter") ?: "Milliliter")

        // Update the count
        binding.fragmentHomeWaterCurrent.text =
            String.format(
                resources.getString(waterUnit.unit),
                (newValue * waterUnit.modifier).toInt()
            )

        // TODO: move it elsewhere
        binding.fragmentHomeWaterTotal.text =
            String.format(
                resources.getString(waterUnit.unit),
                viewModel.dailyWaterIntake
            )

        // get the with/height of the ImageView
        var width = 1500
        var height = 750

        if (binding.background.width != 0) {
            width = binding.background.width
            height = binding.background.height
        }


        // Prepare the update animation
        val animator = ValueAnimator.ofInt(
            this.oldValue.toInt(),
            newValue
        )
        animator.duration = 300 // ms
        animator.addUpdateListener {

            this.oldValue = (it.animatedValue as Int).toFloat()
            val value = 100 * it.animatedValue as Int / viewModel.dailyWaterIntake.toFloat()

            val graph = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
            val canvas = Canvas(graph)
            val rect = RectF(
                10f,
                15f,
                90f,
                85f
            )

            // background Arc
            DrawUtils.drawArc(
                canvas,
                100f,
                rect,
                MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorOnPrimary
                ),
                3f
            )

            // Draw the big Arc
            DrawUtils.drawArc(
                canvas,
                min(max(value, 0.01f), 100f),
                rect,
                MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorPrimary
                ),
                6f
            )

            // save the canvas
            canvas.save()

            // send it
            binding.background.setImageBitmap(graph)
        }

        // start the animation
        animator.start()
    }
}
