package com.dzeio.openhealth.ui.home

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.core.BaseViewModel
import com.dzeio.openhealth.data.step.StepRepository
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.water.WaterRepository
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightRepository
import com.dzeio.openhealth.units.Units
import com.dzeio.openhealth.utils.Configuration
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class HomeViewModel @Inject internal constructor(
    private val weightRepository: WeightRepository,
    private val waterRepository: WaterRepository,
    stepRepository: StepRepository,
    config: Configuration
) : BaseViewModel() {

    private val _steps = MutableLiveData(0)

    /**
     * Steps taken today by the user
     */
    val steps: LiveData<Int> = _steps

    /**
     * Number of steps the use should do today
     */
    val stepsGoal: LiveData<Int?> = config.getInt(Settings.STEPS_GOAL).toLiveData()

    private val _water = MutableLiveData<Water?>(null)

    /**
     * Quantity of water the user drank today
     */
    val water: LiveData<Water?> = _water

    private val _weights = MutableLiveData<List<Weight>?>(null)

    /**
     * The list of weight of the user
     */
    val weights: LiveData<List<Weight>?> = _weights

    /**
     * The size of a cup for the quick water intake add
     */
    var waterCupSize = config.getInt("water_cup_size").toLiveData()

    /**
     * The unit used to display the water intake of the user
     */
    var waterUnit = Units.Volume.find(config.getString("water_unit").value ?: "ml")

    private val _massUnit = MutableLiveData(Units.Mass.KILOGRAM)

    /**
     * The Mass unit used by the user
     */
    val massUnit: LiveData<Units.Mass> = _massUnit

    /**
     * the User weight goal
     */
    val goalWeight = config.getFloat(Settings.WEIGHT_GOAL).toLiveData()

    val dailyWaterIntake: Float = (config.getFloat("water_intake").value ?: 1200f) * waterUnit.modifier

    init {
        // Fetch today's water intake
        viewModelScope.launch {
            waterRepository.todayWater().collectLatest {
                _water.postValue(it)
            }
        }

        // Fetch the user weights
        viewModelScope.launch {
            _steps.postValue(stepRepository.todaySteps())
        }

        // fetch the user weights
        viewModelScope.launch {
            weightRepository.getWeights().collectLatest {
                _weights.postValue(it)
            }
        }

        config.getString(Settings.MASS_UNIT).apply {
            addObserver {
                if (it == null) return@addObserver
                _massUnit.postValue(Units.Mass.find(it))
            }
            if (value != null) {
                _massUnit.postValue(Units.Mass.find(value!!))
            }
        }
    }

    fun updateWater(water: Water) {
        viewModelScope.launch {
            waterRepository.addWater(water)
        }
    }

    fun deleteWater(item: Water) {
        viewModelScope.launch {
            waterRepository.deleteWater(item)
            _water.postValue(null)
        }
    }
}
